/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: FixedFileMapping.java,v 1.2 2005/08/15 16:43:04 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileChannel.MapMode;

class FixedFileMapping implements VirtualFileMappingStrategy {

	private VirtualFile virtualFile;

	public FixedFileMapping() {
		super();
	}

	public void setVirtualFile(VirtualFile file) {
		this.virtualFile = file;
	}

	public MappedByteBuffer readData(long position, int length)
			throws OdetteFTPException {

		MappedByteBuffer recordBuffer = null;
		FileChannel channel = virtualFile.getChannel();

		try {
			recordBuffer = channel.map(MapMode.READ_ONLY, position, length);
		} catch (IOException e) {
			throw new VirtualFileMappingException("Error while mapping the "
					+ "Virtual File record to buffer.", e);
		}

		return recordBuffer;
	}

	public void writeData(long position, ByteBuffer buffer)
			throws OdetteFTPException {

		FileChannel channel = virtualFile.getChannel();

		try {
			channel.write(buffer, position);
		} catch (IOException e) {
			throw new VirtualFileMappingException("Failed to write record "
					+ "into Virtual File.", e);
		}
	}

}
