/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: BufferUtil.java,v 1.6 2005/08/16 12:45:49 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

public class BufferUtil {

	/**
	 * Return the unsigned value of byte from the position specified in a
	 * <code>ByteBuffer</code>.
	 * 
	 * @param bb
	 *            <code>ByteBuffer</code> being used.
	 * @param position
	 *            From which position the byte should be extracted.
	 * @return The unsigned value of a byte in the indicated position
	 *         represented in <code>short</code>.
	 */
	public static short getUnsignedByte(ByteBuffer bb, int position) {
		return ((short) (bb.get(position) & 0xff));
	}

	/**
	 * 
	 * @param bb
	 * @return
	 */
	public static short getUnsignedByte(ByteBuffer bb) {
		return ((short) (bb.get() & 0xff));
	}

	/**
	 * Read the lasting three of the first four octets from the Stream
	 * Transmission Buffer regarding the header length field.
	 * 
	 * @param buffer
	 *            Header buffer.
	 * @return int Length of the Stream Transmission Buffer from the header
	 *         field.
	 */
	public static int parseHeaderLengthField(ByteBuffer buffer) {

		/*
		 * Stream Transmission Buffer length are represented by the lasting 3
		 * header octets. Each octet length value are extracted and than shifted
		 * to left. Make counting with unsigned byte values.
		 */

		int length = 0;
		length += (getUnsignedByte(buffer, 1) << 16); // first length octet
		length += (getUnsignedByte(buffer, 2) << 8); // second length octet
		length += (getUnsignedByte(buffer, 3)); // third length octet

		return length;
	}

	/**
	 * 
	 * @param src
	 * @param position
	 * @param length
	 */
	protected static void putAtPosition(ByteBuffer buffer, byte[] src,
			int position) {

		buffer.position(position);
		buffer.put(src);
	}
}
