/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: Service.java,v 1.12 2005/08/16 03:51:26 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import org.fossilec.odettej.OdetteFTPException;
import org.fossilec.odettej.transport.Transport;

/**
 * 
 * @author Rafael Marins
 */
public abstract class Service {

	/**
	 * <code>Transport</code> class used by this service layer for exchange
	 * raw data.
	 */
	private Transport transport;

	private int exchangeBufferSize;

	/**
	 * Class constructor used with a given transport implementation which should
	 * be used instead of creating a new one. This constructor is almost used
	 * when working as the Responder with this API.
	 * 
	 * @param transport
	 *            <code>Transport</code> implementation used by the service
	 *            layer.
	 * @throws OdetteFTPException
	 * @throws IllegalArgumentException
	 *             If the properties argument is null.
	 * @see org.fossilec.odettej.Session
	 * @see org.fossilec.odettej.transport.Transport
	 */
	protected Service(Transport transport, int debSize) {
		super();
		this.transport = transport;
		this.exchangeBufferSize = debSize;
	}

	public boolean connect(String address) throws OdetteFTPException {
		Transport transport = getTransport();
		return transport.connect(address);
	}

	public void disconnect() throws OdetteFTPException {
		Transport transport = getTransport();
		transport.disconnect();
	}

	protected int getExchangeBufferSize() {
		return exchangeBufferSize;
	}

	public Transport getTransport() throws OdetteFTPException {
		return transport;
	}

	public boolean isConnected() throws OdetteFTPException {
		Transport transport = getTransport();
		return transport.isConnected();
	}

	public abstract OdetteExchangeBuffer receive(boolean strictFormat)
			throws OdetteFTPException;

	public abstract int send(OdetteExchangeBuffer oeb)
			throws OdetteFTPException;

	public static Service getInstance(Transport transport, int debSize,
			boolean useStreamTransmissionBuffer) {

		Service instance;

		if (transport == null) {
			throw new IllegalArgumentException("Transport must not be null");
		}

		if (useStreamTransmissionBuffer) {
			instance = new StreamTransmissionService(transport, debSize);
		} else {
			instance = new UsualService(transport, debSize);
		}

		return instance;
	}

}