/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: NetworkAddress.java,v 1.1 2005/08/15 16:44:36 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.transport;

import java.io.Serializable;

/**
 * This class represents a Network Address with no association to a
 * transportation layer. Subclasses will tipically be associated should be a
 * specific, protocol dependent, implementation.
 * <p>
 * It provides a general model object used throughout API for making
 * connections.
 * 
 * @author Rafael Marins
 */
public abstract class NetworkAddress implements Serializable {

	/**
	 * Protected constructor that allow subclasses to create others
	 * constructors.
	 */
	protected NetworkAddress() {
		super();
	}

	/**
	 * General form of creating a <code>NetworkAddress</code> object using the
	 * String representation of the address.
	 * 
	 * @param address
	 *            String representation of the network address.
	 * @throws AddressException
	 *             Fail to parse the address.
	 */
	protected NetworkAddress(String address) throws AddressException {
		super();
		parse(address);
	}

	/**
	 * Allow the setting up of subclasses context when invoked created through
	 * general constructor <code>NetworkAddress(String)</code>.
	 * 
	 * @param address
	 *            String representation of the network address.
	 * @throws AddressException
	 *             Fail to parse the address.
	 */
	protected abstract void parse(String address) throws AddressException;

	/**
	 * Return a type string that identifies this address type.
	 * 
	 * @return Address type.
	 */
	public abstract String getType();

	/**
	 * Return a String representation of this address object.
	 * 
	 * @return String representation of this address.
	 */
	public abstract String toString();
}
