/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: StreamTransmissionBufferTest.java,v 1.3 2005/08/15 16:45:39 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

import junit.framework.TestCase;

import org.fossilec.odettej.OdetteFTPException;

/**
 * 
 * @author Rafael Marins
 */
public class StreamTransmissionBufferTest extends TestCase {

	/**
	 * Test the StreamTransmissionBuffer(ByteBuffer exchangeBuffer) class
	 * constructor creating the exchanging buffer for a simple command exchange
	 * buffer data (change direction). Thus assert equals while comparing both
	 * header buffers and odette exchange buffers.
	 * 
	 * @throws OdetteFTPException
	 */
	public void testValidExchangeBuffer() throws Exception {

		ByteBuffer exchangeBuffer = TestData.createCommandRawBufferCD();

		// use the StreamTransmissionBuffer subclass with public access methods
		StreamTransmissionBuffer streamBuffer = new StreamTransmissionBuffer(
				exchangeBuffer, true);

		// create the comparing header buffer from the exchange buffer
		// (4 bytes reserved to the header buffer)
		exchangeBuffer.rewind();
		exchangeBuffer.limit(4);

		ByteBuffer header = exchangeBuffer.slice();
		exchangeBuffer.limit(5);

		// both header buffer must have the same contents
		assertEquals(header, streamBuffer.getHeaderBuffer());

		// create the comparing odette exchanging buffer for the CD command
		exchangeBuffer.position(4);
		ByteBuffer oeb = exchangeBuffer.slice();

		// both oeb must have the same contents
		assertEquals(oeb, streamBuffer.getExchangeBuffer().getBuffer());
	}

	public void testInvalidSmallBuffer() throws Exception {

		ByteBuffer rawBuffer = TestData.createCommandRawBufferCD();

		rawBuffer.rewind();
		rawBuffer.limit(3);
		ByteBuffer exchangeBuffer = rawBuffer.slice();

		try {
			// instantiate the class with a small buffer (size=3)
			new StreamTransmissionBuffer(exchangeBuffer, true);
		} catch (BufferCorruptedException e) {
			return;
		} catch (OdetteFTPException e) {
			e.printStackTrace();
		}

		assertFalse(true);
	}

	public void testNewFromOdetteExchangeBuffer() throws Exception {

		// create raw buffer for the CD command
		ByteBuffer rawData = ByteBuffer.allocateDirect(1);
		rawData.put("R".getBytes("US-ASCII"));
		rawData.rewind();

		// create the StreamTransmissionBuffer from a ExchangeBuffer CD Command
		OdetteExchangeBuffer exchangeBuffer = OdetteExchangeBuffer.getInstance(
				rawData, true);

		StreamTransmissionBuffer streamBuffer = new StreamTransmissionBuffer(
				exchangeBuffer);

		// must be 5 (4 header octets + 1 content octet)
		assertEquals(5, streamBuffer.getLength());

		assertEquals(exchangeBuffer, streamBuffer.getExchangeBuffer());
	}
}
